/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus
 * Module  : IPersonDB
 * Package : Examples
 * Author  : Benjamin Hadorn
 * Datum   : $Date$
 * Ablage  : $File$
 * System  : Zeus-Framework
 ***************************************************************************
 * Licence:                                                                *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version             *
 *   2.1 of the License, or (at your option) any later version.            *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA  *
 ***************************************************************************/

/***************************************************************************
 Changes :
 $Log$
***************************************************************************/

#ifndef IPersonDBHPP
#define IPersonDBHPP


#include <zeusbase/System/Interfaces/IXRootObject.hpp>
#include <Interfaces/IPerson.hpp>


BEGIN_NAMESPACE_Zeus

#define INTERFACE_IPersonDB   0x10000003

/****************************************************************************/
/*! This interface defines the access to the persons
*/
/****************************************************************************/
class IPersonDB : public IXRootObject
{
  public:
    /************************************************************************/
    /*! adds a new person to the data base
        \param rName : Name of the person
        \param rLastName : Last name of the person
        \return Index of the person inside the data base
    */
    /************************************************************************/
    virtual long MQUALIFIER addPerson(const IString& rName, const IString& rLastName) = 0;

    /************************************************************************/
    /*! Delete an indexed person
        \retval true : Person deleted
        \retval false: Could not delete this person
    */
    /************************************************************************/
    virtual bool MQUALIFIER deletePerson(long lIndex) = 0;

    /************************************************************************/
    /*! Returns the number of the persons
    */
    /************************************************************************/
    virtual long MQUALIFIER getPersonCount() const=0;

    /************************************************************************/
    /*! returns the indexed person
        \param lIndex : Index of the requested person
        \param rpPerson : Return parameter
        \retval RET_NOERROR : Person returned
        \retval RET_REQUEST_FAILED : Could not find the indexed person.
    */
    /************************************************************************/
    virtual unsigned long MQUALIFIER getPerson(long lIndex, IPerson*& rpPerson)=0;
};

END_NAMESPACE_Zeus

#endif
