/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus
 * Module  : PersonDB
 * Package : Examples
 * Author  : Benjamin Hadorn
 * Datum   : $Date$
 * Ablage  : $File$
 * System  : Zeus-Framework
 ***************************************************************************
 * Licence:                                                                *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version             *
 *   2.1 of the License, or (at your option) any later version.            *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA  *
 ***************************************************************************/

/***************************************************************************
 Changes :
 $Log$
***************************************************************************/
//---------------------------------------------------------------------------

#include "PersonDB.h"
#include "Person.h"
#include <zeusbase/System/XObjectFactory.h>

//---------------------------------------------------------------------------

USING_NAMESPACE_Zeus


/****************************************************************************/
/*! Creates an empty data base object
*/
/****************************************************************************/
TPersonDB::TPersonDB(IXMLNode& rNode, NAMESPACE_Zeus::IXMLDocument& rDocu)
 : TXRootObject(rNode, rDocu)
{}

/****************************************************************************/
/*! Destroys the data base
*/
/****************************************************************************/
TPersonDB::~TPersonDB()
{
  releaseInstances();
}

/****************************************************************************/
/*! Releases all data base entries
*/
/****************************************************************************/
void TPersonDB::releaseInstances()
{
  while(!m_lstPersons.isEmpty())
  {
    m_lstPersons[0]->release();
    m_lstPersons.deleteItem(0);
  }
}

/****************************************************************************/
/*! \see IPersonDB::addPerson
*/
/****************************************************************************/
long MQUALIFIER TPersonDB::addPerson(const IString& rName, const IString& rLastName)
{
  long lRetval = -1;
  long lMaxID;
  this->readAttribute(L"MaxID", lMaxID);
  lMaxID++;

  //First create a new node
  IXMLNode* pChildNode = NULL;
  if (m_pNode->createNewChildNode(TYPE_NODE_ELEMENT, TString(L"Person"), &TString(), pChildNode) == RET_NOERROR)
  {
    pChildNode->setAttributeValue(TString(L"ClassName"), TString(L"TPerson"));
    pChildNode->setAttributeValue(TString(L"ID"), TString(lMaxID));
    pChildNode->setAttributeValue(TString(L"LastName"), rLastName);
    pChildNode->setAttributeValue(TString(L"Name"), rName);

    //then use the x-object factory to create the x-object
    IXObject* pChildObj = NULL;
    if (MyXObjectFactory.addNewChildObjects(*pChildNode, *this, pChildObj, true, NULL) == RET_NOERROR)
    {
      lRetval = m_lstPersons.getCount()-1;
      pChildObj->release();
    }
    pChildNode->release();
  }

  //store the max id
  this->storeAttribute(L"MaxID", lMaxID);
  return lRetval;
}

/****************************************************************************/
/*! \see IPersonDB::deletePerson
*/
/****************************************************************************/
bool MQUALIFIER TPersonDB::deletePerson(long lIndex)
{
  bool bRetval = false;
  IPerson* pPerson = m_lstPersons[lIndex];
  if (pPerson != NULL)
  {
    IXObject* pObject = NULL;
    if (pPerson->askForInterface(INTERFACE_IXObject, ICAST(pObject)) == RET_NOERROR)
    {
      pObject->freeze();
      this->removeChild(*pObject);
      pObject->release();
    }
    bRetval = true;
  }
  return bRetval;
}

/****************************************************************************/
/*! \see IPersonDB::getPersonCount
*/
/****************************************************************************/
long MQUALIFIER TPersonDB::getPersonCount() const
{
  return m_lstPersons.getCount();
}

/****************************************************************************/
/*! \see IPersonDB::getPerson
*/
/****************************************************************************/
unsigned long MQUALIFIER TPersonDB::getPerson(long lIndex, IPerson*& rpPerson)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  rpPerson = m_lstPersons[lIndex];
  if (rpPerson != NULL)
  {
    rpPerson->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/****************************************************************************/
/*! \see TXObject::onChildAdded
*/
/****************************************************************************/
void TPersonDB::onChildAdded(IXObject& child)
{
  IPerson* pPerson = NULL;
  if (child.askForInterface(INTERFACE_IPerson, ICAST(pPerson)) == RET_NOERROR)
  {
    m_lstPersons.add(pPerson);
  }
}

/****************************************************************************/
/*! \see TXObject::onChildAdded
*/
/****************************************************************************/
void TPersonDB::onChildDeleted(IXObject& child)
{
  IPerson* pPerson = NULL;
  if (child.askForInterface(INTERFACE_IPerson, ICAST(pPerson)) == RET_NOERROR)
  {
    long lIndex = m_lstPersons.indexOf(pPerson);
    if (lIndex != -1)
    {
      m_lstPersons[lIndex]->release();
      m_lstPersons.deleteItem(lIndex);
    }
    pPerson->release();
  }
}



/****************************************************************************/
/*! Memory management
*/
/****************************************************************************/
MEMORY_MANAGER_IMPL(TPersonDB);
  INTERFACE_CAST(IPersonDB, INTERFACE_IPersonDB);
MEMORY_MANAGER_IMPL_PARENT_END(TXRootObject);

