/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus
 * Module  : PersonDB
 * Package : Examples
 * Author  : Benjamin Hadorn
 * Datum   : $Date$
 * Ablage  : $File$
 * System  : Zeus-Framework
 ***************************************************************************
 * Licence:                                                                *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version             *
 *   2.1 of the License, or (at your option) any later version.            *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA  *
 ***************************************************************************/

/***************************************************************************
 Changes :
 $Log$
***************************************************************************/
//---------------------------------------------------------------------------

#ifndef PersonDBH
#define PersonDBH
//---------------------------------------------------------------------------

#include <Interfaces/IPersonDB.hpp>
#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/SingleLinkedList.hpp>

BEGIN_NAMESPACE_Zeus

//---------------------------------------------------------------------------
/****************************************************************************/
/*! Implementation of the data base
*/
/****************************************************************************/
class TPersonDB : public TXRootObject, public IPersonDB
{
  public:
    TPersonDB(IXMLNode& rNode, NAMESPACE_Zeus::IXMLDocument& rDocu);

    //Methods of IPersonDB
    virtual long MQUALIFIER addPerson(const IString& rName, const IString& rLastName);
    virtual bool MQUALIFIER deletePerson(long lIndex);
    virtual long MQUALIFIER getPersonCount() const;
    virtual unsigned long MQUALIFIER getPerson(long lIndex, IPerson*& rpPerson);

    //Methods of IXRootObject
    virtual unsigned long MQUALIFIER save() { return TXRootObject::save(); }
    virtual unsigned long MQUALIFIER saveAs(const IString& rFileName) { return TXRootObject::saveAs(rFileName); }


    REG_ROOT_BEGIN(TPersonDB)
    REG_ROOT_ADD(TPersonDB, L"TPersonDB");
    REG_ROOT_END

    //Memory management
    MEMORY_MANAGER_DECL

  protected:
    virtual ~TPersonDB();

    virtual void onChildAdded(IXObject& child);
    virtual void onChildDeleted(IXObject& child);

  private:
    ///List of all persons
    TSingleLinkedList<IPerson*> m_lstPersons;

    void releaseInstances();

};


END_NAMESPACE_Zeus

#endif
