/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 ***************************************************************************
 * Projekt : Zeus
 * Module  : frmMainForm
 * Package : Examples
 * Author  : Benjamin Hadorn
 * Datum   : $Date$
 * Ablage  : $File$
 * System  : Zeus-Framework
 ***************************************************************************
 * Licence:                                                                *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version             *
 *   2.1 of the License, or (at your option) any later version.            *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU Lesser General Public License for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA  *
 ***************************************************************************/

/***************************************************************************
 Changes :
 $Log$
***************************************************************************/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "frmMainForm.h"
#include <PersonDB.h>
#include <zeusbase/System/String.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/Int.h>


//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ExtCtrls"
#pragma link "StdCtrls"
#pragma link "Dialogs"
#pragma resource "*.dfm"

USING_NAMESPACE_Zeus

TForm1 *Form1;


/***************************************************************************/
/*! Creates a new main form
*/
/***************************************************************************/
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
  m_pPersonDB = NULL;
  newDB();
  m_lIndex = 0;
  showEntry(m_lIndex);
}

/***************************************************************************/
/*! Destroys the main form
*/
/***************************************************************************/
__fastcall TForm1::~TForm1()
{
  releaseInstances();
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TForm1::releaseInstances()
{
  closeDB();
}

/***************************************************************************/
/*! opens a data base
    \param rFileName : Name of the data base
    \retval true : Data base opened
    \retval false: Could not open the data base
*/
/***************************************************************************/
bool TForm1::openDB(const IString& rFileName)
{
  bool bRetval = false;
  closeDB();

  IXObject* pRootObj = NULL;
  if (MyXObjectFactory.createObjectFromFile(rFileName, pRootObj, false, true) == RET_NOERROR)
  {
    if (pRootObj->askForInterface(INTERFACE_IPersonDB, ICAST(m_pPersonDB)) == RET_NOERROR)
    {
      m_strFileName = rFileName;
      bRetval = true;
    }
    else
    {
      MessageBox(NULL, "Could not create the personal DB.", "Error", MB_OK);
    }
    pRootObj->release();
  }
  else
  {
    MessageBox(NULL, "No personal DB found in file", "Error", MB_OK);
  }

  if (bRetval)
  {
    showEntry(m_lIndex);
  }


  return bRetval;
}

/***************************************************************************/
/*! Save the data base to file
*/
/***************************************************************************/
bool TForm1::saveDB()
{
  bool bRetval = false;

  if (m_strFileName.isEmpty())
  {
    if (m_dlgSave->Execute())
    {
      m_strFileName = m_dlgSave->FileName.c_str();
      m_strFileName = m_strFileName.terminated(TString(L".xml"));
    }
  }

  //Save procedure
  if (!m_strFileName.isEmpty())
  {
    m_lbFilename->Caption = m_strFileName.c_bstr();

    if (m_pPersonDB == NULL)
    {
      newDB();
    }

    m_pPersonDB->saveAs(m_strFileName);
    bRetval = true;
  }
  return bRetval;
}


/***************************************************************************/
/*! closes the data base
*/
/***************************************************************************/
void TForm1::closeDB()
{
  if (m_pPersonDB != NULL)
  {
    m_pPersonDB->release();
    m_pPersonDB = NULL;
  }

  //Show empty
  m_strFileName = L"";
  m_lIndex = 0;
  showEntry(m_lIndex);
  m_lbFilename->Caption = L"<none>";
}

/***************************************************************************/
/*! creates a new data base
*/
/***************************************************************************/
void TForm1::newDB()
{
  closeDB();
  TString strXML = L"<?xml version='1.0' encoding='ISO-8859-1' standalone='yes'?>\n\
<DB ClassName=\"TPersonDB\"/>";

  IXObject* pRootObj = NULL;
  if (MyXObjectFactory.createObjectFromStream(strXML, pRootObj, false, true) == RET_NOERROR)
  {
    if (pRootObj->askForInterface(INTERFACE_IPersonDB, ICAST(m_pPersonDB)) != RET_NOERROR)
    {
      MessageBox(NULL, "Could not create the data base file", "Error", MB_OK);
    }
    pRootObj->release();
  }
}



/***************************************************************************/
/*! Shows an indexed data base entry
    \param lIndex: Index of the entry
*/
/***************************************************************************/
void TForm1::showEntry(long& lIndex)
{
  bool bOK = false;

  if (m_pPersonDB != NULL)
  {
    IPerson* pPerson = NULL;
    if (m_pPersonDB->getPerson(lIndex, pPerson) == RET_NOERROR)
    {
      TString strName;
      pPerson->getFirstName(strName);
      m_edtVorname->Text = strName.c_bstr();

      pPerson->getLastName(strName);
      m_edtName->Text = strName.c_bstr();

      m_lbIDOut->Caption = TInt(pPerson->getPersonID()).toHex().c_bstr();

      pPerson->release();
      bOK = true;
    }

    long lCount = m_pPersonDB->getPersonCount();
    if (lIndex >= lCount)
    {
      lIndex = lCount;
    }
    if (lIndex < 0)
    {
      lIndex = 0;
    }

    m_btnNext->Enabled = (lIndex < lCount-1);
    m_btnPrev->Enabled = (lIndex > 0);
    m_btnSet->Enabled = (lIndex < lCount);
    m_btnDelete->Enabled = (lIndex < lCount);

  }

  //clear endit fields
  if (!bOK)
  {
    m_edtName->Text = L"";
    m_edtVorname->Text = L"";
    m_lbIDOut->Caption = L"0";
  }
}

/***************************************************************************/
/*! Opens a new data base form file. This will show the open dialog
*/
/***************************************************************************/
void __fastcall TForm1::Button1Click(TObject*)
{
  if (m_dlgOpen->Execute())
  {
    TString strFileName = m_dlgOpen->FileName.c_str();
    bool bOpen = openDB(strFileName);

    if (bOpen)
    {
      m_lbFilename->Caption = strFileName.c_bstr();
    }
    else
    {
      m_lbFilename->Caption = L"<none>";
    }
  }
}
//---------------------------------------------------------------------------

/***************************************************************************/
/*! Save button pressed
*/
/***************************************************************************/
void __fastcall TForm1::m_btnSaveClick(TObject*)
{
  saveDB();
}
//---------------------------------------------------------------------------

/***************************************************************************/
/*! Previous button pressed
*/
/***************************************************************************/
void __fastcall TForm1::m_btnPrevClick(TObject*)
{
  --m_lIndex;
  showEntry(m_lIndex);
}
//---------------------------------------------------------------------------

/***************************************************************************/
/*! Next button pressed
*/
/***************************************************************************/
void __fastcall TForm1::m_btnNextClick(TObject*)
{
  ++m_lIndex;
  showEntry(m_lIndex);

}
//---------------------------------------------------------------------------

/***************************************************************************/
/*! Add button pressed
*/
/***************************************************************************/
void __fastcall TForm1::Button3Click(TObject*)
{
  if (m_pPersonDB != NULL)
  {
    m_lIndex = m_pPersonDB->addPerson(TString(m_edtVorname->Text.c_str()), TString(m_edtName->Text.c_str()));
    showEntry(m_lIndex);
  }
}
//---------------------------------------------------------------------------

/***************************************************************************/
/*! Set button pressed
*/
/***************************************************************************/
void __fastcall TForm1::m_btnSetClick(TObject*)
{
  IPerson* pPerson = NULL;
  if (m_pPersonDB != NULL && m_pPersonDB->getPerson(m_lIndex, pPerson) == RET_NOERROR)
  {
    pPerson->setLastName(TString(m_edtName->Text.c_str()));
    pPerson->setFirstName(TString(m_edtVorname->Text.c_str()));
    pPerson->release();
  }
}

/***************************************************************************/
/*! Delete button pressed
*/
/***************************************************************************/
void __fastcall TForm1::m_btnDeleteClick(TObject*)
{
  if (m_pPersonDB != NULL)
  {
    m_pPersonDB->deletePerson(m_lIndex);
    if (m_lIndex > m_pPersonDB->getPersonCount() -1)
    {
      m_lIndex = m_pPersonDB->getPersonCount() -1;
    }
    showEntry(m_lIndex);
  }
}
//---------------------------------------------------------------------------

